@@ -75,7 +75,7 @@ t_http_header *parse_http_headers(char *line) {
 		if ((value = strchr(http_header->data, ':')) != NULL) {
 			do {
 				value++;
-			} while ((*value == ' ') && (*value != '\0'));
+			} while (*value == ' ');
 			http_header->value_offset = (value - http_header->data);
 		} else {
 			http_header->value_offset = 0;
@@ -106,28 +106,6 @@ char *get_http_header(char *key, t_http_header *http_headers) {
 	return NULL;
 }
 
-/* Reset the strlen of the HTTP header
- */
-void reset_http_header_strlen(char *key, t_http_header *http_headers) {
-	int len;
-
-	if ((key == NULL) || (http_headers == NULL)) {
-		return;
-	}
-
-	len = strlen(key);
-	while (http_headers != NULL) {
-		if (strncasecmp(http_headers->data, key, len) == 0) {
-			http_headers->length = strlen(http_headers->data);
-			if (http_headers->value_offset > http_headers->length) {
-				http_headers->value_offset = 0;
-			}
-			break;
-		}
-		http_headers = http_headers->next;
-	}
-}
-
 /* free() a list of http_headers.
  */
 t_http_header *remove_http_headers(t_http_header *http_headers) {
@@ -404,11 +382,14 @@ int parse_keyvalue(char *line, t_keyvalue **kvlist, char *delimiter) {
 			free(*kvlist);
 			return -1;
 		}
+		(*kvlist)->key_len = strlen((*kvlist)->key);
+
 		if (((*kvlist)->value = strdup(remove_spaces(value))) == NULL) {
 			free((*kvlist)->key);
 			free(*kvlist);
 			return -1;
 		}
+		(*kvlist)->value_len = strlen((*kvlist)->value);
 	} else {
 		return -1;
 	}
